package com.sd.cavphmi.bindadapters

import android.graphics.Typeface
import android.text.SpannableStringBuilder
import android.text.Spanned
import android.text.style.RelativeSizeSpan
import android.text.style.StyleSpan
import android.widget.ImageView
import android.widget.TextView
import androidx.databinding.BindingAdapter
import com.sd.cavphmi.R
import java.text.DecimalFormat
import kotlin.math.absoluteValue

object ShowCarStatuObject {


    /**信号***/
    @JvmStatic
    @BindingAdapter("showXinhao")
    fun showXinhao(iv: ImageView, timeDiff: Int) {
        when (timeDiff) {
            in 1..100 -> iv.setImageResource(R.drawable.xinhao5)
            in 101..200 -> iv.setImageResource(R.drawable.xinhao4)
            in 201..300 -> iv.setImageResource(R.drawable.xinhao3)
            in 301..400 -> iv.setImageResource(R.drawable.xinhao2)
            else -> {
                iv.setImageResource(R.drawable.xinhao1)
            }
        }
    }

    /**泊车状态***/
    @JvmStatic
    @BindingAdapter("showAvpStatus")
    fun showAvpStatus(tv: TextView, avpStatus: Int?) {
        when (avpStatus) {
            1 -> tv.setText("场外行驶")
            2, 6 -> tv.setText("场内行驶")
            3 -> tv.setText("自动泊车中")
            4 -> tv.setText("车辆停靠")
            5 -> tv.setText("自动召车中")
            else -> {
                tv.setText("")
            }
        }
    }

    @JvmStatic
    @BindingAdapter("showSpeed")
    fun showSpeed(tv: TextView, speed: Float) {
        var sb = SpannableStringBuilder()

        var mixSpan = RelativeSizeSpan(1f)
        var maxSpan = RelativeSizeSpan(2f)
        var bold = StyleSpan(Typeface.BOLD)

        if (speed > 0f) {
            val df: DecimalFormat = DecimalFormat("#.0")
            val s: String = df.format(speed.times(3.6f))
            sb.append("${s}km/h")
            sb.setSpan(maxSpan, 0, s.count(), Spanned.SPAN_INCLUSIVE_EXCLUSIVE)
            sb.setSpan(bold, 0, s.count(), Spanned.SPAN_INCLUSIVE_EXCLUSIVE)
            sb.setSpan(mixSpan, s.count(), sb.count(), Spanned.SPAN_INCLUSIVE_EXCLUSIVE)
            tv.setText(sb)
        } else {
            sb = SpannableStringBuilder("0km/h")
            sb.setSpan(maxSpan, 0, 1, Spanned.SPAN_INCLUSIVE_EXCLUSIVE)
            sb.setSpan(bold, 0, 1, Spanned.SPAN_INCLUSIVE_EXCLUSIVE)
            sb.setSpan(mixSpan, 1, sb.count(), Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
            tv.setText(sb)
        }
    }

    @JvmStatic
    @BindingAdapter("showHeading")
    fun showHeading(tv: TextView, head: Int) {
        var heading = head.absoluteValue
        when (heading) {
            0 -> {
                tv.setText("正北")
            }

            90 -> {
                tv.setText("正东")
            }

            180 -> {
                tv.setText("正南")
            }

            270 -> {
                tv.setText("正北")
            }

            in 1..45 -> {
                tv.setText("北偏东${heading}°")
            }

            in 46..89 -> {
                tv.setText("东偏北${90-heading}°")
            }

            in 91..135 -> {
                tv.setText("东偏南${heading-90}°")
            }

            in 136..179 -> {
                tv.setText("南偏东${180 - heading}°")
            }

            in 181..225 -> {
                tv.setText("南偏西${heading - 180}°")
            }

            in 226..269 -> {
                tv.setText("西偏南${270 - heading}°")
            }

            in 271..315 -> {
                tv.setText("西偏北${heading - 270}°")
            }

            in 316..359 -> {
                tv.setText("北偏西${360 - heading}°")
            }

            else -> {
                tv.setText(" - -")
            }
        }
    }


}