package com.sd.cavphmi.ui

import android.view.KeyEvent
import androidx.activity.viewModels
import androidx.lifecycle.ViewModelProvider
import com.sd.cavphmi.BR
import com.sd.cavphmi.R
import com.sd.cavphmi.base.BaseActivity
import com.sd.cavphmi.bean.CarVehicle
import com.sd.cavphmi.bean.ParkBean
import com.sd.cavphmi.bean.VehicleStats
import com.sd.cavphmi.databinding.ActivityMainBinding
import com.sd.cavphmi.intfaces.OnConCan
import com.sd.cavphmi.viewmodels.AvpMapVM
import com.sd.cavphmi.viewmodels.MainVm
import com.sd.cavphmi.viewmodels.MockVM
import dagger.hilt.android.AndroidEntryPoint


@AndroidEntryPoint
class MainActivity : BaseActivity<ActivityMainBinding, MainVm>() {

    override fun getStatuBarColor(): Int {
        return -1
    }

    override fun initContentView(): Int {
        return R.layout.activity_main
    }

    override fun initViewModel(): MainVm {
        return ViewModelProvider(this).get(MainVm::class.java)
    }

    override fun initVariableId(): Int {
        return BR.vm
    }

    private val mockVM: MockVM by viewModels()
    private val avpMapVM: AvpMapVM by viewModels()

    private lateinit var dialogFragment: ExitAppDialog

    override fun initView() {
        viewModel.mockVM = mockVM

    }

    override fun getToData() {
        viewModel.startWS()
        viewModel.getCurrentTime()
        viewModel.getMobileNetworkSignal()
        viewModel.getOrderData()
        viewModel.getSpaceDataLoc()
//        viewModel.getSpaceData()
        //test
//        viewModel.findPathPlanning()
    }

    override fun initListener() {
        mockBt()
        mockLis()
        //订单数据
        viewModel.orderBean.observe(this) {
            binding.order = it
        }
    }

    private fun mockLis() {

    }

    private fun mockBt() {
        binding.btVehicle.setOnClickListener {
            viewModel.subVehicle().observe(this) {
                topicVehicle(it)
            }
        }
        binding.btStatus.setOnClickListener {
            viewModel.subVehicleStatus().observe(this) {
//                println("time = ${time}")
                topicVehicleStatus(it)
            }
        }
        binding.btTarget.setOnClickListener {
            viewModel.subTarget()
        }
        binding.btV2x.setOnClickListener {
            viewModel.subStartV2x()
        }
        binding.btV2xend.setOnClickListener {
            viewModel.subEndV2x()
        }
        binding.btChewei.setOnClickListener {
            viewModel.subPoints().observe(this) {
                topicvPoints(it)
            }
        }
        binding.btAllline.setOnClickListener {
            viewModel.subLinePlaning()
        }
        binding.btJubuline.setOnClickListener {
            viewModel.subPartLine()
        }
        binding.btRsi.setOnClickListener {
            viewModel.subRsiEvent()
        }
    }

    /**车辆姿态**/
    private fun topicVehicle(bean: CarVehicle) {

    }

    /**联网车辆状态数据**/
    private fun topicVehicleStatus(bean: VehicleStats) {
//        if ((bean.vehicleData.count()) > 0) {

//        }
    }

    /**停车位变更**/
    private fun topicvPoints(bean: ParkBean) {
        var curValue = bean.parkingSpace
//
        if (viewModel.spaceCode.isNotEmpty()) {
//            // 更新停车位的信息
            var parkingSpot = viewModel.getParkingSpot(curValue)
//            this.$refs.basisCesium.updateParkingSpace(parkingSpot);
        } else {
//            // 第一次需要添加停车位的效果
//            let carPostion = [baseData.mainLocation.lon, baseData.mainLocation.lat];
//            // 开启前进自主泊车效果
//            let parkingSpot = this.getParkingSpot(curValue);
//            this.$refs.basisCesium.addForwardParkingEffect({
//                    id: baseData.mainCarID,
//                    carPostion: carPostion,
//                    spaceInfo: parkingSpot,
//            });
        }
//
//        // 记录 spaceCode 给倒车入库的时候用
        viewModel.spaceCode = curValue
//
//        this.highlightSpaceEntity(spaceCode)
    }


    override fun dispatchKeyEvent(event: KeyEvent): Boolean {
        if ((event.getKeyCode() == KeyEvent.KEYCODE_BACK) || event.getKeyCode() == KeyEvent.KEYCODE_HOME) {
//            var ft = supportFragmentManager.beginTransaction()
            dialogFragment = ExitAppDialog()
            dialogFragment.onConCan = object : OnConCan {
                override fun onCon() {
                    viewModel.cleanRes()
                    finish()
                }

                override fun onCan() {
                }
            }
            dialogFragment.show(supportFragmentManager, "exit")
            return true
        } else {
            return super.dispatchKeyEvent(event)
        }
    }
}