package com.sd.cavphmi.viewmodels

import android.content.Context
import androidx.databinding.ObservableField
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import com.sd.cavphmi.base.MyBaseViewModel
import com.sd.cavphmi.bean.CarVehicle
import com.sd.cavphmi.bean.LinePlaningBean
import com.sd.cavphmi.bean.OrderBean
import com.sd.cavphmi.bean.ParkBean
import com.sd.cavphmi.bean.PartLineBean
import com.sd.cavphmi.bean.PerTarget
import com.sd.cavphmi.bean.ReqBean
import com.sd.cavphmi.bean.RsiEventBean
import com.sd.cavphmi.bean.SpaceDataLocBeanItem
import com.sd.cavphmi.bean.SpaceNoBean
import com.sd.cavphmi.bean.VToXImgBean
import com.sd.cavphmi.bean.VehicleStats
import com.sd.cavphmi.bean.WebSetBean
import com.sd.cavphmi.net.MyResult
import com.sd.cavphmi.repositorys.ParseSocketRepo
import com.sd.cavphmi.utils.DateUtils
import com.sd.cavphmi.utils.FileIoUtils
import com.sd.cavphmi.utils.MyContants
import com.sd.cavphmi.utils.ToastHelper
import com.sd.cavphmi.websockets.MyWebSocketClient
import dagger.hilt.android.lifecycle.HiltViewModel
import dagger.hilt.android.qualifiers.ApplicationContext
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import org.java_websocket.handshake.ServerHandshake
import protocol.Response
import java.lang.reflect.Type
import java.net.URI
import java.util.Timer
import java.util.TimerTask
import javax.inject.Inject


@HiltViewModel
class MainVm @Inject constructor(
    @ApplicationContext private var context: Context,
    private var parseSocketRepo: ParseSocketRepo
//    var spaceNoRepo: SpaceNoRepo,
) : MyBaseViewModel() {

    private var client: MyWebSocketClient? = null

    private var gson = Gson()

    //模拟操作
    lateinit var mockVM: MockVM

    //是否需要模拟
    var isMock = false

    //车辆位姿数据
    var carVehicle = MutableLiveData<CarVehicle>()

    //联网车辆状态数据topic
    var vehicleStat = MutableLiveData<VehicleStats>()

    //感知目标物
    var targetPre = MutableLiveData<PerTarget>()

    //停车位变更
    var parkBean = MutableLiveData<ParkBean>()

    //停车位编码
    var spaceCode = ""

    //订单信息
    var orderBean = MutableLiveData<OrderBean>()

    //老停车状态，用于判断状态切换
//    private var oldAvpStatus = -1

    // 是否处理路径规划显示的路径
    private var needDealPath = false

    //驾驶模式
    var driveMode = ObservableField("")

    //航向角
    var heading = ObservableField(550)

    //速度
    var speed = ObservableField(0f)

    //当前时间
    var curTime = ObservableField("")

    //网络延时时间差
    var timeOutDiff = ObservableField(-1)

    // 当前时间
    private var curTimer = Timer()

    // 信号强度
    private var netSingTimer = Timer()

    // 订单详情
    private var orderTimer = Timer()

    // 车位信息
    private var carSpaNoTimer = Timer()


    /***获取当前时间**/
    fun getCurrentTime() {
        curTimer.schedule(object : TimerTask() {
            override fun run() {
                var time = DateUtils.getCurrentTime12HourFormat()
                curTime.set(time)
            }
        }, 2000, 1000 * 60)
    }

    /***获取信号强度**/
    fun getMobileNetworkSignal() {
        netSingTimer.schedule(object : TimerTask() {
            override fun run() {
                viewModelScope.launch {
                    var result = parseSocketRepo.getTimeOut()
                    when (result) {
                        is MyResult.Success<Int> -> {
                            timeOutDiff.set(result.data)
//                            println("-------时间差---- = ${result.data}")
                        }

                        else -> {
//                        println("-------Error---- = ${result.msg}")
                        }
                    }
                }
            }
        }, 5000, 1000 * 60)
    }

    /***获取订单信息
     * @param vehiclePlate 车牌号
     * **/
    fun getOrderData() {
        orderTimer.schedule(object : TimerTask() {
            override fun run() {
//                var vehiclePlate = carVehicle.value?.vehicleData?.get(0)?.vehiclePlate ?: ""
//                if (TextUtils.isEmpty(vehiclePlate)) {
//                    return
//                }
                var vehiclePlate = "吉AC242"
                viewModelScope.launch {
                    var result =
                        parseSocketRepo.getOrderData(vehiclePlate)
                    when (result) {
                        is MyResult.Success<OrderBean> -> {
                            println("-------订单信息---- = ${result.data}")
                            FileIoUtils.writeToFile(gson.toJson(result.data), "order.txt")
                            orderBean.value = result.data
                            var avpStatus = result.data.avpStatus
//                            var plateNo = result.data.plateNo
//                            var plateNo = "吉AC242"
                            if (avpStatus == 3) {//泊车
                                startParkingVehicle(vehiclePlate)
                            } else if (avpStatus == 5) {//招车
                                startRecruitmentVehicle(vehiclePlate)
                            } else {
                                endParkingVehicle() // 结束泊车
                                endRecruitmentVehicle()//// 由招车转为其他状态
                            }
                        }
//                        if (oldAvpStatus != avpStatus) {
//                            if (avpStatus != 3 && oldAvpStatus == 3) {
////                                    endParkingVehicle() // 结束泊车
//                            } else if (avpStatus == 3) {
//                                startParkingVehicle(plateNo) // 开始泊车
//                            }
//                            if (avpStatus != 5 && oldAvpStatus == 5) {
//
//                                endRecruitmentVehicle()
//                            } else if (avpStatus == 5) {
//                                // 招车需要显示路径
//                                startRecruitmentVehicle(plateNo)
//                            }
//                            oldAvpStatus = avpStatus
//                        }
                        is MyResult.Error -> {
                            println("-------订单信息 ERROR---- = ${result}")
                        }
                    }
//                FileIoUtils.writeToFile(gson.toJson(bean), "carvehicle.txt")
                }
            }
        }, 3000, 1000 * 50)
    }

    /**开始泊车*/
    private fun startParkingVehicle(plateNo: String) {
        // worker中处理路径规划显示的路径
        needDealPath = true
        // 获取规划好的路径
        findPathPlanning(plateNo)
    }

    /**
     * 结束泊车
     */
    private fun endParkingVehicle() {
        // 移除自主泊车所有效果
//        this.$refs.basisCesium.removeParkingEffect();
//
//        // 停车位信息重置
//        spaceCode = "";
//        this.highlightSpaceEntity(spaceCode)
//
//        // 是否处理路径规划显示的路径
//        this.needDealPath = false;
//
//        // 小地图上不显示线
//        cesiumMainIns.updateLinePosition2D(polyine2d, {
//            before: [],
//            after: [],
//            left: [],
//            right: []
//        });
        // console.log('window.leftPath', window.leftPath)
    }

    /*** 开始招车，显示路径规划路径*/
    private fun startRecruitmentVehicle(vehiclePlate: String) {
        // worker不处理路径规划显示的路径
        this.needDealPath = true
        // 获取规划好的路径
        findPathPlanning(vehiclePlate, false)
    }

    /** 由招车转为其他状态，移除路径规划路线*/
    private fun endRecruitmentVehicle() {
        // worker中不处理路径规划显示的路径
        this.needDealPath = false;
//
//        // 移除路面上的路线
//        this.$refs.basisCesium.removePathPlanLine();
//
//        // 小地图上不显示线
//        cesiumMainIns.updateLinePosition2D(polyine2d, {
//            before: [],
//            after: [],
//            left: [],
//            right: []
//        });
    }

    /**
     * 获取路径规划
     */
    fun findPathPlanning(vehiclePlate: String, ifParking: Boolean = true) {
        viewModelScope.launch {
            var result = parseSocketRepo.getLinePlaning(vehiclePlate)
            FileIoUtils.writeToFile(gson.toJson(result), "http_path.txt")
            when (result) {
                is MyResult.Success -> {
                    var lins = result.data.line
                    if (lins.isNotEmpty()) {
//                        this.$refs.basisCesium.findPathPlanning(res);
//                        // 泊车需要高亮车位，召车不高亮
                        if (ifParking) {
                            spaceCode = result.data.spaceCode
                        } else {
//                            spaceCode = ''
                        }
//                        this.highlightSpaceEntity(spaceCode)
                    }
                }

                else -> {
                    println("-----获取路径规划 = ${result}")
                }
            }
        }
    }


    /**查询车位信息 判断车位上是否有车**/
    fun getSpaceData() {
        carSpaNoTimer.schedule(object : TimerTask() {
            override fun run() {
                viewModelScope.launch {
                    var result = parseSocketRepo.getSpaceData()
                    when (result) {
                        is MyResult.Success<SpaceNoBean> -> {
                            println("-------------查询车位信息= ${result}")
                            FileIoUtils.writeToFile(gson.toJson(result.data), "spaceno.txt")
                        }

                        is MyResult.Error -> {
                            println("-------------查询车位信息  Error= ${result}")
//                            carSpaNoTimer.cancel()
                        }
                    }
                }
            }
        }, 4000, 1000 * 20)
    }

    /**获取本地车位**/
    fun getSpaceDataLoc() {
        viewModelScope.launch {
            var str = FileIoUtils.getAssetContent(context, "SpaceJSON.json")
            var ty = object : TypeToken<List<SpaceDataLocBeanItem>>() {

            }.type
            var carNo = gson.fromJson<List<SpaceDataLocBeanItem>>(str, ty)
        }
    }

    /**
     * 根据停车位编号，获取停车位的具体信息
     * @param {*} parkingValue
     */
    fun getParkingSpot(parkingValue: String) {
        viewModelScope.launch {
            withContext(Dispatchers.IO) {
//               // 查询倒车入库的车位
//               const parkingSpot = SpaceData.find(
//                       (spot) => spot.spaceCode === parkingValue
//               );
//               return parkingSpot;
            }
        }
    }


    fun startWS() {
//        val httpHeaders = mutableMapOf<String, String>()
//        httpHeaders.put("Cookie", "username=nemo")
        try {
            client = MyWebSocketClient(URI(MyContants.WS_HOST))
            client?.onSocketCb = onSocketCb
            client?.onDataCb = onDataCb
            client?.connect()
        } catch (e: Exception) {

        }
    }

    /***清理资源***/
    fun cleanRes() {
        curTimer.cancel()
        netSingTimer.cancel()
        orderTimer.cancel()
        carSpaNoTimer.cancel()
        closeWS()
    }


    private fun closeWS() {
        if (client?.isOpen ?: false) {
            client?.close()
        }
    }


    private var onSocketCb = object : MyWebSocketClient.OnSocketCb {
        override fun onOpen(handshakedata: ServerHandshake?) {
        }

        override fun onClose(code: Int, reason: String?, remote: Boolean) {
        }

        override fun onError(ex: Exception?) {
        }
    }


    private var onDataCb = object : MyWebSocketClient.OnDataCb {
        override fun onVehicle(res: Response.SocketResponse) {
            viewModelScope.launch {
                FileIoUtils.writeToFile(res.data, "CarVehicle.txt")
                val bean =
                    parseSocketRepo.parseDataBean(res.data, CarVehicle::class.java)
                println("------车辆位姿数据 = ${res.data}")
                if ((bean.vehicleData.count()) > 0) {
                    heading.set(bean.vehicleData.get(0).heading.toInt())
                    speed.set(bean.vehicleData.get(0).speed)
                }
            }
        }

        override fun onVehicleStats(res: Response.SocketResponse) {
            viewModelScope.launch {
                FileIoUtils.writeToFile(res.data, "vehicleS.txt")
                val bean =
                    parseSocketRepo.parseDataBean(res.data, VehicleStats::class.java)
                println("------联网车辆状态数据 = ${res.data}")
                if (bean.vehicleStatusList.isNotEmpty()) {
                    val str = when (bean.vehicleStatusList[0].driveMode) {
                        0 -> "自动驾驶"
                        1 -> "人工驾驶"
                        2 -> "远程驾驶"
                        else -> "—— ——"
                    }
                    driveMode.set(str)
                }
                vehicleStat.value = bean
            }
        }

        override fun onTarget(res: Response.SocketResponse) {
            viewModelScope.launch {
                FileIoUtils.writeToFile(res.data, "PerTarget.txt")
                var bean = parseSocketRepo.parseDataBean(res.data, PerTarget::class.java)
                println("-----感知目标物数据 = ${bean}")
                targetPre.value = bean
            }
        }

        override fun onStartV2x(res: Response.SocketResponse) {
            viewModelScope.launch {
                println("-----V2X预警开始 = ${res.data}")
                FileIoUtils.writeToFile(res.data, "onStartV2x.txt")
            }
        }

        override fun onEndV2x(res: Response.SocketResponse) {
            println("--------V2X预警结束--${res.data}")
            viewModelScope.launch {
                FileIoUtils.writeToFile(res.data, "onEndV2x.txt")
            }
        }

        override fun onPoint(res: Response.SocketResponse) {
            viewModelScope.launch {
                println("-----停车位变更 = ${res.data}")
                FileIoUtils.writeToFile(res.data, "change_park.txt")
                var bean = parseSocketRepo.parseDataBean(res.data, ParkBean::class.java)
                parkBean.value = bean
            }
        }

        override fun onLinePlaning(res: Response.SocketResponse) {
            viewModelScope.launch {
                println("-----全局路径发生切换 = ${res.data}")
                FileIoUtils.writeToFile(res.data, "all_line.txt")
                var bean = parseSocketRepo.parseDataBean(res.data, LinePlaningBean::class.java)
//                parkBean.value = bean

//                FileIoUtils.writeToFile(gson.toJson(bean), "line_plan.txt")
            }
        }

        override fun onPartLine(res: Response.SocketResponse) {
            viewModelScope.launch {
                println("-----局部路径 = ${res.data}")
                FileIoUtils.writeToFile(res.data, "part_line.txt")
                var bean = parseSocketRepo.parseDataBean(res.data, PartLineBean::class.java)
//                parkBean.value = bean
//                FileIoUtils.writeToFile(gson.toJson(bean), "part_line.txt")
            }
        }

        override fun onRsiEvent(res: Response.SocketResponse) {
            viewModelScope.launch {
                println("-----rsi事件 = ${res.data}")
                var bean = parseSocketRepo.parseDataBean(res.data, RsiEventBean::class.java)

//                parkBean.value = bean
//                FileIoUtils.writeToFile(gson.toJson(bean), "rsi_event.txt")
                FileIoUtils.writeToFile(res.data, "rsi_event.txt")
            }
        }
    }


    /***  联网车辆位姿数据**/
    fun subVehicle(): LiveData<CarVehicle> {
        if (isMock) {
            mockVM.onVehicleMock(carVehicle, heading, speed)
        } else {
            client?.send(getSendData("/topic/vehicle"))
        }
        return carVehicle
    }

    /**  联网车辆状态数据
     *
     * {"vehicleStats":{"total":0,"avp":0},"vehicleStatusList":[],"wsTime":1750138623719}
     * */
    fun subVehicleStatus(): LiveData<VehicleStats> {
        val reqStr = getSendData("/topic/vehicleStatus")
        client?.send(reqStr)
        return vehicleStat
    }

    /**感知目标物数据
     * {"objectData":[{"altitude":0.0,"deviceCode":"ALL","id":"rsm","latitude":0.0,"longitude":0.0,"participant":[],"reportTime":1750138879684}],"wsTime":1750138834204}
     * **/
    fun subTarget() {
        val reqStr = getSendData("/topic/dataMerge")
        client?.send(reqStr)
    }

    /**V2X预警开始
     *
     * **/
    fun subStartV2x() {
        val reqStr = getSendData("/topic/v2xStart")
        client?.send(reqStr)
    }

    /*** V2X预警结束topic**/
    fun subEndV2x() {
        var reqStr = getSendData("/topic/v2xEnd")
        client?.send(reqStr)
    }

    /*** 更新停车位**/
    fun subPoints(): LiveData<ParkBean> {
        val reqStr = getSendData("/topic/points")
        client?.send(reqStr)
        return parkBean
    }

    /*** 全局路径发生切换**/
    fun subLinePlaning() {
        val reqStr = getSendData("/topic/linePlaning")
        client?.send(reqStr)
    }

    /*** 局部路径**/
    fun subPartLine() {
        val reqStr = getSendData("/topic/partLine")
        client?.send(reqStr)
    }

    /*** rsi事件**/
    fun subRsiEvent() {
        val reqStr = getSendData("/topic/rsiEvent")
        client?.send(reqStr)
    }

    /**构建发送体**/
    private fun getSendData(key: String): String {
        var data = WebSetBean.subDic.get(key)
        var req = ReqBean()
        req.data = data!!
        var str = gson.toJson(req)
        return str
    }


    fun showVToTip() {
        var imgs = VToXImgBean.imgs
        ToastHelper.showCustViewShort(context, imgs.get("2")!!)
    }


}